#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "xmlparser.h"

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent)
    , ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    
    QMenu* pmnu = new QMenu("&Menu");
    pmnu->addAction("&Open",
                    this,
                    SLOT(openXmlFile()),
                    Qt::CTRL + Qt::Key_O
                    );
    pmnu->addAction("Close all",
                    this,
                    SLOT(closeAll()),
                    Qt::CTRL + Qt::SHIFT + Qt::Key_W
                    );
    pmnu->addAction("&Exit",
                    this,
                    SLOT(close()),
                    Qt::CTRL + Qt::Key_Q
                    );
    this->menuBar()->addMenu(pmnu);
    


    query_te = new QTextEdit(this);
    query_te->setFixedHeight(150);
    run_btn = new QPushButton("Run", this);
    run_btn->setFixedSize(150, 50);
    chooseFile_btn = new QPushButton("Choose File", this);
    chooseFile_btn->setFixedSize(150, 50);
    treeView = new QTreeView(this);
    enterQuery_lbl = new QLabel("Enter query:", this);


    btns_vLayout = new QVBoxLayout(this);
    btns_vLayout->setSpacing(50);
    btns_vLayout->addWidget(run_btn);
    btns_vLayout->addWidget(chooseFile_btn);

    query_hLayout = new QHBoxLayout(this);
    query_hLayout->addWidget(query_te);
    query_hLayout->addLayout(btns_vLayout);

    treeView->setHeaderHidden(true);
    main_vLayout = new QVBoxLayout(this);
    main_vLayout->addWidget(treeView);
    main_vLayout->addWidget(enterQuery_lbl);
    main_vLayout->addLayout(query_hLayout);

    centralWidget()->setLayout(main_vLayout);
    filePaths = new QStringList;
    filePaths->reserve(30);
    handler = nullptr;
    treeView->setContextMenuPolicy(Qt::CustomContextMenu);
    connect(chooseFile_btn, SIGNAL(clicked()),
            this, SLOT(openXqFile()));
    connect(run_btn, SIGNAL(clicked()),
            this, SLOT(on_run_btn_pressed()));
    connect(treeView, SIGNAL(clicked(QModelIndex)),
            this, SLOT(treeViewClicked(QModelIndex)));
    connect(treeView, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(ContextMenu(QPoint)));

}

bool MainWindow::openXmlFile()
{
    QString path = QFileDialog::getOpenFileName(this, "Open File", "", "*.xml");
    if(path.isEmpty())
        return false;
    if(handler == nullptr)
    {
        handler = new XMLParser;
        handler->treeWidget = new QTreeWidget();
    }
    handler->title = path.section('/', -1);
    treeView->setModel(handler->treeWidget->model());
    QFile             file(path);
    QXmlInputSource   source(&file);
    QXmlSimpleReader  reader;

    reader.setContentHandler(handler);
    reader.parse(source);
    filePaths->push_back(path);
    return true;
}

void MainWindow::closeAll()
{
    handler->treeWidget->clear();
}

bool MainWindow::openXqFile()
{
    QString xqPath = QFileDialog::getOpenFileName(this, "Open File", "", "*.xq");
    if(!activeIndex.isValid())
    {
        QMessageBox::warning(this, "Error",
                             "You have not selected an active file");
        return false;
    }
    QString xmlPath = *(filePaths->begin() + activeIndex.row());

    QMessageBox::warning(this, "Error",
                         xmlPath);
    if(xqPath == "")
        return false;
    QString strQuery = "";
    QFile xqFile(xqPath);
    if(xqFile.open(QIODevice::ReadOnly))
    {
        strQuery = xqFile.readAll();
        xqFile.close();
    }
    else
    {
        QMessageBox::warning(this, "Error",
                             "Can't open the file for reading: " + xqPath);
        return false;
    }

    QFile xmlFile(xmlPath);
    if(xmlFile.open(QIODevice::ReadOnly))
    {
        QXmlQuery query;
        query.bindVariable("inputDocument", &xmlFile);
        query.setQuery(strQuery);
        if (!query.isValid())
        {
            QMessageBox::warning(this, "Error",
                                 "The query is invalid");
            return false;
        }

        QString strOutput;
        if (!query.evaluateTo(&strOutput))
        {
            QMessageBox::warning(this, "Error",
                                 "Can't evaluate the query");
            return false;
        }

        xmlFile.close();
        query_te->setText(strOutput);
    }


    return true;
}

void MainWindow::on_run_btn_pressed()
{
    if(!activeIndex.isValid())
    {
        QMessageBox::warning(this, "Error",
                             "You have not selected an active file");
        return;
    }

    QString queryStr = query_te->toPlainText();

    QString str = *(filePaths->begin() + activeIndex.row());
    QFile xmlFile(str);
    QXmlQuery query;
    if (xmlFile.open(QIODevice::ReadOnly))
    {
        query.bindVariable("inputDocument", &xmlFile);
        query.setQuery(queryStr);
        if (!query.isValid())
        {
            QMessageBox::warning(this, "Error",
                                 "The query is invalid");
            return;
        }

        QStringList listOutput;
        QString strOutput;
        if (!query.evaluateTo(&listOutput))
        {
            QMessageBox::warning(this, "Error",
                                 "Can't evaluate the query");
            return;
        }
        xmlFile.close();
        QListWidget* list_widget = new QListWidget;
        for (auto it : listOutput)
        {
            list_widget->addItem(it);
        }
        QListView* list_view = new QListView;
        list_view->setModel(list_widget->model());
        QDockWidget* docWgt = new QDockWidget;
        docWgt->setMinimumSize(500, 200);
        docWgt->setWidget(list_view);
        docWgt->setWindowTitle("XQuery Window");
        docWgt->show();
    }
}

void MainWindow::ContextMenu(QPoint pos)
{
    auto idx = treeView->currentIndex();
    auto parent = idx.parent();
    if(!parent.isValid())
    {
        QMenu* menu = new QMenu(this);

        menu->popup(treeView->viewport()->mapToGlobal(pos));
        QAction *active = new QAction("Set as Active",this);
        menu->addAction(active);
        connect(active, SIGNAL(triggered()), this, SLOT(setFileActive()));
        QAction* close = new QAction("Close", this);
        menu->addAction(close);
        connect(close, SIGNAL(triggered()), this, SLOT(slotCloseFile()));

    }
}
void MainWindow::setFileActive()
{
    QModelIndex index = treeView->currentIndex();

    QFont fontStyle;
    fontStyle.setBold(false);
    treeView->model()->setData(activeIndex, fontStyle, Qt::FontRole);
    fontStyle.setBold(true);
    treeView->model()->setData(index, fontStyle, Qt::FontRole);
    activeIndex = index;
}

void MainWindow::slotCloseFile()
{
    QModelIndex index = treeView->currentIndex();
    treeView->model()->removeRow(index.row());
    filePaths->removeAt(index.row());
}

MainWindow::~MainWindow()
{
    delete ui;
}

